@@ -1,8 +1,7 @@
 #' UI for the sign in and register pages
 #'
 #' UI for the sign in and register pages when a user invite is required to register and
-#' sign in.  See \code{\link{sign_in_no_invite_module}} if you do not require your
-#' users to sign in and register to access your 'shiny' app.
+#' sign in.
 #'
 #' @param id the Shiny module id
 #' @param register_link The text that will be displayed in the link to go to the
@@ -11,7 +10,7 @@
 #'
 #' @importFrom shiny textInput actionButton NS actionLink
 #' @importFrom htmltools tagList tags div h1 br hr
-#' @importFrom shinyFeedback useShinyFeedback
+#' @importFrom shinyFeedback useShinyFeedback loadingButton
 #' @importFrom shinyjs useShinyjs hidden
 #'
 #' @export
@@ -23,12 +22,101 @@ sign_in_module_ui <- function(
 ) {
   ns <- shiny::NS(id)
 
-  firebase_config <- .global_sessions$firebase_config
   providers <- .global_sessions$sign_in_providers
 
+  continue_sign_in <- div(
+    id = ns("continue_sign_in"),
+    shiny::actionButton(
+      inputId = ns("submit_continue_sign_in"),
+      label = "Continue",
+      width = "100%",
+      class = "btn btn-primary btn-lg"
+    )
+  )
 
+  sign_in_password_ui <- div(
+    id = ns("sign_in_password_ui"),
+    div(
+      class = "form-group",
+      style = "width: 100%;",
+      tags$label(
+        tagList(icon("unlock-alt"), "password"),
+        `for` = "sign_in_password"
+      ),
+      tags$input(
+        id = ns("sign_in_password"),
+        type = "password",
+        class = "form-control",
+        value = ""
+      )
+    ),
+    br(),
+    shinyFeedback::loadingButton(
+      ns("sign_in_submit"),
+      label = "Sign In",
+      class = "btn btn-primary btn-lg text-center",
+      style = "width: 100%",
+      loadingLabel = "Authenticating...",
+      loadingClass = "btn btn-primary btn-lg text-center",
+      loadingStyle = "width: 100%"
+    )
+  )
 
+  continue_registration <- div(
+    id = ns("continue_registration"),
+    shiny::actionButton(
+      inputId = ns("submit_continue_register"),
+      label = "Continue",
+      width = "100%",
+      class = "btn btn-primary btn-lg"
+    )
+  )
 
+  register_passwords <- div(
+    id = ns("register_passwords"),
+    div(
+      class = "form-group",
+      style = "width: 100%",
+      tags$label(
+        tagList(icon("unlock-alt"), "password"),
+        `for` = ns("register_password")
+      ),
+      tags$input(
+        id = ns("register_password"),
+        type = "password",
+        class = "form-control",
+        value = ""
+      )
+    ),
+    br(),
+    div(
+      class = "form-group shiny-input-container",
+      style = "width: 100%",
+      tags$label(
+        tagList(shiny::icon("unlock-alt"), "verify password"),
+        `for` = ns("register_password_verify")
+      ),
+      tags$input(
+        id = ns("register_password_verify"),
+        type = "password",
+        class = "form-control",
+        value = ""
+      )
+    ),
+    br(),
+    div(
+      style = "text-align: center;",
+      shinyFeedback::loadingButton(
+        ns("register_submit"),
+        label = "Register",
+        class = "btn btn-primary btn-lg",
+        style = "width: 100%;",
+        loadingLabel = "Registering...",
+        loadingClass = "btn btn-primary btn-lg text-center",
+        loadingStyle = "width: 100%;"
+      )
+    )
+  )
 
   email_ui <- tags$div(
     id = ns("email_ui"),
@@ -38,66 +126,22 @@ sign_in_module_ui <- function(
         class = "text-center",
         style = "padding-top: 0;",
         "Sign In"
-      )
-    ),
-    shinyjs::hidden(div(
-      id = ns("register_panel_top"),
-      h1(
-        class = "text-center",
-        style = "padding-top: 0;",
-        "Register"
       ),
-      br()
-    )),
-
-
-    tags$br(),
-    email_input(
-      inputId = ns("email"),
-      label = tagList(icon("envelope"), "email"),
-      value = ""
+      tags$br(),
+      email_input(
+        inputId = ns("sign_in_email"),
+        label = tagList(icon("envelope"), "email"),
+        value = ""
+      ),
+      tags$br()
     ),
-    tags$br(),
-
-
     tags$div(
       id = ns("sign_in_panel_bottom"),
-      shinyjs::hidden(div(
-        id = ns("sign_in_password"),
-        div(
-          class = "form-group",
-          style = "width: 100%;",
-          tags$label(
-            tagList(icon("unlock-alt"), "password"),
-            `for` = "password"
-          ),
-          tags$input(
-            id = ns("password"),
-            type = "password",
-            class = "form-control",
-            value = ""
-          )
-        ),
-        br(),
-        shinyFeedback::loadingButton(
-          ns("submit_sign_in"),
-          label = "Sign In",
-          class = "btn btn-primary btn-lg text-center",
-          style = "width: 100%",
-          loadingLabel = "Authenticating...",
-          loadingClass = "btn btn-primary btn-lg text-center",
-          loadingStyle = "width: 100%"
-        )
-      )),
-      div(
-        id = ns("continue_sign_in"),
-        shiny::actionButton(
-          inputId = ns("submit_continue_sign_in"),
-          label = "Continue",
-          width = "100%",
-          class = "btn btn-primary btn-lg"
-        )
-      ),
+      if (isTRUE(.global_sessions$is_invite_required)) {
+        tagList(continue_sign_in, shinyjs::hidden(sign_in_password_ui))
+      } else {
+        sign_in_password_ui
+      },
       div(
         style = "text-align: center;",
         if (is.null(register_link)) {
@@ -112,73 +156,33 @@ sign_in_module_ui <- function(
           )
         },
         br(),
-        tags$button(
-          class = 'btn btn-link btn-small',
-          id = ns("reset_password"),
-          "Forgot your password?"
-        )
+        send_password_reset_email_module_ui(ns("reset_password"))
       )
     ),
 
+    shinyjs::hidden(div(
+      id = ns("register_panel_top"),
+      h1(
+        class = "text-center",
+        style = "padding-top: 0;",
+        "Register"
+      ),
+      tags$br(),
+      email_input(
+        inputId = ns("register_email"),
+        label = tagList(icon("envelope"), "email"),
+        value = ""
+      ),
+      tags$br()
+    )),
 
     shinyjs::hidden(div(
       id = ns("register_panel_bottom"),
-      div(
-        id = ns("continue_registation"),
-        br(),
-        shiny::actionButton(
-          inputId = ns("submit_continue_register"),
-          label = "Continue",
-          width = "100%",
-          class = "btn btn-primary btn-lg"
-        )
-      ),
-      shinyjs::hidden(div(
-        id = ns("register_passwords"),
-        div(
-          class = "form-group",
-          style = "width: 100%",
-          tags$label(
-            tagList(icon("unlock-alt"), "password"),
-            `for` = ns("register_password")
-          ),
-          tags$input(
-            id = ns("register_password"),
-            type = "password",
-            class = "form-control",
-            value = ""
-          )
-        ),
-        br(),
-        div(
-          class = "form-group shiny-input-container",
-          style = "width: 100%",
-          tags$label(
-            tagList(shiny::icon("unlock-alt"), "verify password"),
-            `for` = ns("register_password_verify")
-          ),
-          tags$input(
-            id = ns("register_password_verify"),
-            type = "password",
-            class = "form-control",
-            value = ""
-          )
-        ),
-        br(),
-        br(),
-        div(
-          style = "text-align: center;",
-          shinyFeedback::loadingButton(
-            ns("submit_register"),
-            label = "Register",
-            class = "btn btn-primary btn-lg",
-            style = "width: 100%;",
-            loadingLabel = "Registering...",
-            loadingClass = "btn btn-primary btn-lg text-center",
-            loadingStyle = "width: 100%;"
-          )
-        )
-      )),
+      if (isTRUE(.global_sessions$is_invite_required)) {
+        tagList(continue_registration, shinyjs::hidden(register_passwords))
+      } else {
+        register_passwords
+      },
       div(
         style = "text-align: center",
         hr(),
@@ -214,19 +218,13 @@ sign_in_module_ui <- function(
 
   htmltools::tagList(
     shinyjs::useShinyjs(),
-    shinyFeedback::useShinyFeedback(feedback = FALSE),
     tags$div(
       class = "auth_panel",
       ui_out
     ),
-    firebase_dependencies(),
-    firebase_init(firebase_config),
-    tags$script(src = "polish/js/toast_options.js"),
-    tags$script(src = "polish/js/auth_all.js?version=1"),
-    tags$script(paste0("auth_all('", ns(''), "')")),
-    tags$script(src = "https://cdn.jsdelivr.net/npm/js-cookie@2/src/js.cookie.min.js"),
-    tags$script(src = "polish/js/auth_firebase.js?version=6"),
-    tags$script(paste0("auth_firebase('", ns(''), "')"))
+    tags$script(src = "polish/js/auth_keypress.js?version=4"),
+    tags$script(paste0("auth_keypress('", ns(''), "')")),
+    sign_in_js(ns)
   )
 }
 
@@ -238,12 +236,14 @@ sign_in_module_ui <- function(
 #' @param output the Shiny output
 #' @param session the Shiny session
 #'
-#' @importFrom shiny observeEvent observe getQueryString
-#' @importFrom shinyFeedback showToast resetLoadingButton
+#' @importFrom shiny observeEvent observe getQueryString updateTextInput
 #' @importFrom shinyjs show hide
 #' @importFrom shinyWidgets sendSweetAlert
+#' @importFrom shinyFeedback showToast
 #' @importFrom digest digest
+#' @importFrom httr POST authenticate
 #'
+#' @export
 #'
 sign_in_module <- function(input, output, session) {
   ns <- session$ns
@@ -263,6 +263,17 @@ sign_in_module <- function(input, output, session) {
     shinyjs::show("register_panel_bottom")
   }
 
+  shiny::observeEvent(input$go_to_register, {
+    go_to_registration_page()
+  })
+
+  shiny::callModule(
+    send_password_reset_email_module,
+    "reset_password",
+    email = reactive({input$sign_in_email})
+  )
+
+
   # if query parameter "register" == TRUE, then go directly to registration page
   observe({
     query_string <- shiny::getQueryString()
@@ -275,7 +286,7 @@ sign_in_module <- function(input, output, session) {
 
   shiny::observeEvent(input$submit_continue_sign_in, {
 
-    email <- tolower(input$email)
+    email <- tolower(input$sign_in_email)
 
     # check user invite
     invite <- NULL
@@ -291,32 +302,32 @@ sign_in_module <- function(input, output, session) {
           text = "You must have an invite to access this app",
           type = "error"
         )
-        return()
+
       } else {
 
-        # check is user is already registered with Firebase.  If user already registered,
-        # then allow them to continue signing in.  If not registered, take the user
-        # to the registration page and open the passwords to continue registration.
 
-        # this custom message triggers the `input$check_registered_res` input which
-        # will fire off the next observeEvent
-        session$sendCustomMessage(
-          session$ns("check_registered"),
-          message = list(
-            email = email
-          )
+        # user is invited, so continue the sign in process
+        shinyjs::hide("submit_continue_sign_in")
+
+        shinyjs::show(
+          "sign_in_password_ui",
+          anim = TRUE
         )
 
+        # NEED to sleep this exact amount to allow animation (above) to show w/o bug
+        Sys.sleep(.25)
+
+        shinyjs::runjs(paste0("$('#", ns('sign_in_password'), "').focus()"))
+
       }
 
-
-    }, error = function(e) {
+    }, error = function(err) {
       # user is not invited
-      print(e)
+      print(err)
       shinyWidgets::sendSweetAlert(
         session,
         title = "Error",
-        text = "Error checking invite",
+        text = err$message,
         type = "error"
       )
 
@@ -325,54 +336,8 @@ sign_in_module <- function(input, output, session) {
   })
 
 
-  observeEvent(input$check_registered_res, {
-    hold_email <- tolower(input$email)
-
-    is_registered <- input$check_registered_res
-
-    if (isTRUE(is_registered)) {
-      # user is already registered, so continue sign in
-      # user is invited
-      shinyjs::hide("submit_continue_sign_in")
-
-      shinyjs::show(
-        "sign_in_password",
-        anim = TRUE
-      )
-
-      # NEED to sleep this exact amount to allow animation (above) to show w/o bug
-      Sys.sleep(.25)
-
-      shinyjs::runjs(paste0("$('#", ns('password'), "').focus()"))
-
-    } else if (isFALSE(is_registered)) {
-
-      # go to the user registration page
-      go_to_registration_page()
-
-      # open the passwords to continue user registration
-      submit_continue_register_rv(submit_continue_register_rv() + 1)
-    } else {
-
-      print(is_registered)
-      shinyWidgets::sendSweetAlert(
-        session,
-        title = "Error",
-        text = "Error checking invite",
-        type = "error"
-      )
 
 
-    }
-
-
-
-  })
-
-
-  shiny::observeEvent(input$go_to_register, {
-    go_to_registration_page()
-  })
 
   shiny::observeEvent(input$go_to_sign_in, {
     shinyjs::hide("register_panel_top")
@@ -393,7 +358,7 @@ sign_in_module <- function(input, output, session) {
 
   shiny::observeEvent(submit_continue_register_rv(), {
 
-    email <- tolower(input$email)
+    email <- tolower(input$register_email)
 
     invite <- NULL
     tryCatch({
@@ -411,7 +376,7 @@ sign_in_module <- function(input, output, session) {
       }
 
       # user is invited
-      shinyjs::hide("continue_registation")
+      shinyjs::hide("continue_registration")
 
       shinyjs::show(
         "register_passwords",
@@ -436,42 +401,37 @@ sign_in_module <- function(input, output, session) {
 
   }, ignoreInit = TRUE)
 
-  observeEvent(input$check_jwt, {
-    email <- tolower(input$email)
+
+  observeEvent(input$register_js, {
+    hold_email <- input$register_js$email
+    hold_password <- input$register_js$password
+    cookie <- input$register_js$cookie
+
+    hashed_cookie <- digest::digest(cookie)
+
 
     tryCatch({
-
-      # user is invited, so attempt sign in
-      new_user <- .global_sessions$sign_in(
-        input$check_jwt$jwt,
-        digest::digest(input$check_jwt$cookie)
+      .global_sessions$register_email(
+        hold_email,
+        hold_password,
+        hashed_cookie
       )
 
-      if (is.null(new_user)) {
-        shinyFeedback::resetLoadingButton('submit_sign_in')
-        # show unable to sign in message
-        shinyFeedback::showToast('error', 'sign in error')
-        stop('sign_in_module: sign in error', call. = FALSE)
+      remove_query_string()
+      session$reload()
+    }, error = function(err) {
 
-      } else {
-        # sign in success
-        remove_query_string()
-        session$reload()
-      }
-
-    }, error = function(e) {
-      shinyFeedback::resetLoadingButton('submit_sign_in')
-      print(e)
-      shinyWidgets::sendSweetAlert(
-        session,
-        title = "Not Authorized",
-        text = "You must have an invite to access this app",
-        type = "error"
-      )
+      shinyFeedback::resetLoadingButton('register_submit')
 
+      print(err)
+      shinyFeedback::showToast("error", err$message)
     })
 
-
-
   })
+
+  sign_in_check_jwt(
+    jwt = shiny::reactive({input$check_jwt})
+  )
+
+  invisible()
 }
